define('de_epages/product/ui/variationwizard-unity-test-helper',[
    'jquery',

    '$ready!'
], function($) {
    return function() {
        var enterValue = function(input, value, timeout) {
                return function(cb) {
                    input.val(value).trigger('input');

                    setTimeout(function() {
                        cb();
                    }, ((timeout && (typeof timeout).toLowerCase() === 'number') ? timeout : 300));
                };
            },
            toggleCheckbox = function(input, state, timeout) {
                return function(cb) {
                    input.prop('checked', state).trigger('change');

                    setTimeout(function() {
                        cb();
                    }, ((timeout && (typeof timeout).toLowerCase() === 'number') ? timeout : 300));
                };
            },
            clickElement = function(element, timeout) {
                return function(cb) {
                    element.trigger('click');

                    setTimeout(function() {
                        cb();
                    }, ((timeout && (typeof timeout).toLowerCase() === 'number') ? timeout : 300));
                };
            };

        return {
            selectDefault: function(id) {
                var input = $('.ep-variationassistant-attributevaluestable').find('tbody').children().eq(id).find('input');

                return toggleCheckbox(input, true, 100);
            },
            setVarAlias: function(id, string) {
                var input = $('.ep-variationassistant-newvariationalias').eq(id);

                return enterValue(input, string, 300);
            },
            toggleVariation: function(id, state) {
                var input = $('.ep-variationassistant-selectnewvariation').eq(id);

                return toggleCheckbox(input, (state !== undefined ? state : !input.prop('checked')), 300);
            },
            toggleAttribute: function(id, state){
                var input = $('.ep-variationassistant-isusedcheckbox > input[type="checkbox"]').eq(id);

                return toggleCheckbox(input, (state !== undefined ? state : !input.prop('checked')), 300);
            },
            testAttributeActive: function(state){
                var active = $('.ep-variationassistant-isusedcheckbox > input[type="checkbox"]').map(function(index, element) {return !$(element).prop('disabled');}).get();

                QUnit.equal(JSON.stringify(active), JSON.stringify(state.map(function(bool) {return !!bool;})),  'Checkboxes are active as expected.');
            },
            setValue: function(attrId, valueId, string) {
                var input = $('.ep-variationassistant-attributesection').children().eq(attrId)
                                .find('.ep-variationassistant-attributelist').children().eq(valueId).find('input[type="text"]');

                return enterValue(input, string, 300);
            },
            addValue: function(attrId, string) {
                var input = $('.ep-variationassistant-attributesection').children().eq(attrId).find('.ep-variationassistant-attributevalueinput');

                return enterValue(input, string, 300);
            },
            getValue: function(attrId, valueId) {
                var input = $('.ep-variationassistant-attributesection').children().eq(attrId)
                    .find('.ep-variationassistant-attributelist').children().eq(valueId).find('input[type="text"]');

                return input.val();
            },
            deleteValue: function(attrId, valueId) {
                var icon = $('.ep-variationassistant-attributesection').children().eq(attrId)
                                .find('.ep-variationassistant-attributelist').children().eq(valueId).find('.fa-trash');

                return clickElement(icon, 300);

            },
            setSelectionStateAttributeValue: function(attrId, valueId, state) {
                var input = $('.ep-variationassistant-attributesection').children().eq(attrId)
                                .find('.ep-variationassistant-attributelist').children().eq(valueId).find('input[type="checkbox"]');

                return (input.prop('checked') !== state) ? clickElement(input, 100) : function (cb) {
                    cb();
                };
            },
            setAlias: function(attrId, string) {
                var input = $('.ep-variationassistant-attributesection').children().eq(attrId).find('.ep-variationassistant-attributealias');

                return enterValue(input, string, 300);
            },
            getAlias: function(attrId) {
                var input = $('.ep-variationassistant-attributesection').children().eq(attrId).find('.ep-variationassistant-attributealias');

                return input.val();
            },
            addAttribute: function() {
                return clickElement($('#EnterNewAttributeButton'), 50);
            },
            deleteAttribute: function(attrId) {
                var deleteIcon = $('.ep-variationassistant-attributesection').children().eq(attrId).find('.ep-variationassistant-deleteAttribute');

                return clickElement(deleteIcon, 50);
            },
            nextStep: function() {
                return clickElement($('#ep-variationassistant-nextButton'), 1000);
            },
            lastStep: function() {
                return clickElement($('#ep-variationassistant-backButton'), 1000);
            },
            testAliases: function(count) {
                var aliasInputs = $('#ProductTable').find('tbody > tr').find('.ep-input-style-disabled, .ep-variationassistant-newvariationalias'),
                    newAliases = aliasInputs.get().map(function(input, index) {
                        return 'MyAlias-' + '0000'.substring(('' + (index + 1)).length) + (index + 1);
                    }),
                    existingProducts = $('#EnterAttributesForm').productUiVariationwizardUnity('option').existingProducts,
                    aliases = existingProducts.map(function(prod) {
                        return prod.alias;
                    }).concat(newAliases),
                    aliasesValid = aliasInputs.get().every(function(element, index) {
                        var $elem = $(element),
                            value = $elem.is('span') ? $elem.html() : $elem.val();

                        return (aliases.indexOf(value) !== -1);
                    });

                QUnit.ok(aliasesValid, 'All aliases are valid');
                QUnit.equal(aliasInputs.length, count, 'Correct number of variations');
                QUnit.equal($('#ProductTable').find('tbody > tr').find('.ep-input-style-disabled').length, existingProducts.length, 'Correct number of existing products.');
            },
            testButtons: function(opts) {
                var state = $.extend({}, {next: [0, 0], back: [0,0], finish: [0, 0]}, opts);

                // test for error
                if (
                        $('#ep-variationassistant-nextButton').prop('disabled') == !state.next[0]
                        && $('#ep-variationassistant-nextButton').hasClass('HideElement') == !state.next[1]
                        && $('#ep-variationassistant-backButton').prop('disabled') == !state.back[0]
                        && $('#ep-variationassistant-backButton').hasClass('HideElement') == !state.back[1]
                        && $('#ep-variationassistant-finishButton').prop('disabled') == !state.finish[0]
                        && $('#ep-variationassistant-finishButton').hasClass('HideElement') == !state.finish[1]
                    ) {
                    // if no error
                    QUnit.ok(true, 'button state okay');
                } else {
                    // otherwise show details
                    QUnit.equal($('#ep-variationassistant-nextButton').prop('disabled'), !state.next[0],  'Next button is ' + (state.next[0] ? '' : 'not ') + 'enabled');
                    QUnit.equal($('#ep-variationassistant-nextButton').hasClass('HideElement'), !state.next[1], 'Next button is ' + (state.next[1] ? '' : 'not ') + 'visible');
                    QUnit.equal($('#ep-variationassistant-backButton').prop('disabled'), !state.back[0], 'Back button is ' + (state.back[0] ? '' : 'not ') + 'enabled');
                    QUnit.equal($('#ep-variationassistant-backButton').hasClass('HideElement'), !state.back[1], 'Back button is ' + (state.back[1] ? '' : 'not ') + 'visible');
                    QUnit.equal($('#ep-variationassistant-finishButton').prop('disabled'), !state.finish[0], 'Finish button is ' + (state.finish[0] ? '' : 'not ') + 'enabled');
                    QUnit.equal($('#ep-variationassistant-finishButton').hasClass('HideElement'), !state.finish[1], 'Finish button is ' + (state.finish[1] ? '' : 'not ') + 'visible');
                }
            }
        };
    };
});