/*global define*/
/*jslint vars:true, browser:true*/
define([
    'testsuite/qunit',
    'jquery',
    'ep',
    'de_epages/ebay/inc/ebaytools',

    '$ready!'
], function (QUnit, $, ep, ebaytools) {
    'use strict';

    var products = [{
                    "epid": "219540209",
                    "gtin": [
                        "357030843X"
                    ],
                    "brand": "Dyson",
                    "mpn": [
                        "204720-01"
                    ],
                    "image": {
                        "imageUrl": "https://i.ebayimg.com/00/s/NTY4WDEwMDA=/z/TDEAAOSwJQdW-mXz/$_6.JPG?set_id=89040003C1"
                    },
                    "aspects": [{
                            "localizedName": "Brand",
                            "localizedValues": [
                                "Dyson"
                            ]
                        },
                        {
                            "localizedName": "Model",
                            "localizedValues": [
                                "DC58"
                            ]
                        },
                        {
                            "localizedName": "MPN",
                            "localizedValues": [
                                "20472001"
                            ]
                        },
                        {
                            "localizedName": "Type",
                            "localizedValues": [
                                "Handheld"
                            ]
                        },
                        {
                            "localizedName": "Filters",
                            "localizedValues": [
                                "HEPA"
                            ]
                        }
                    ],
                    "ean": [
                        "0885609001920"
                    ],
                    "upc": [
                        "0885609001920"
                    ],
                    "productHref": "https://api.sandbox.ebay.com/commerce/catalog/v1_beta/product/219540209",
                    "title": "Dyson DC58 - Purple/Silver - Handheld Cleaner",
                    "productWebUrl": "https://www.ebay.com/p/Dyson-DC58-Purple-Silver-Handheld-Cleaner/219540209?src=urllib"
                },
                {
                    "epid": "1232143",
                    "gtin": ["0400100836590"],
                    "brand": "Knoedelschleuder",
                    "mpn": ["204720-01"],
                    "image": {
                        "imageUrl": "https://i.ebayimg.com/00/s/NTY4WDEwMDA=/z/TDEAAOSwJQdW-mXz/$_6.JPG?set_id=89040003C1"
                    },
                    "aspects": [{
                        "localizedName": "Brand",
                        "localizedValues": ["Knoedelschleuder"]
                    }, {
                        "localizedName": "Type",
                        "localizedValues": ["Nahrungergaenzugsmittel"]
                    }],
                    "ean": ["0885609001920"],
                    "upc": ["0885609001920"],
                    "productHref": "https://api.sandbox.ebay.com/commerce/catalog/v1_beta/product/219540209",
                    "title": "tolle Knoedelschleuder",
                    "productWebUrl": "https://www.ebay.com/p/Dyson-DC58-Purple-Silver-Handheld-Cleaner/219540209?src=urllib"
                }
            ],

        responseData = {
            "success": true,
            "products": JSON.stringify(products)
        },

        errorResponseData = {
            "success": false
        };

    QUnit.module("de_epages/ebay/ui/ebayassignmentcontroller", {
        // setup: fixture.init
        // ,
        // teardown: fixture.destroy
    });


    QUnit.asyncTest("widget test presence of properties", 1, function () {

        $.each([
            'searchEbayByGtin'
        ], function (i, fn) {
            QUnit.equal(
                ebaytools[fn] !== undefined,
                true,
                'Property "' + fn + '" is defined.'
            );
        });

        QUnit.start();
    });

    QUnit.asyncTest("test modules", function () {

        /* START - fake ajax request */
        ep.ajax = function (o) {
            var result = $.Deferred(),
                resolveSend;

            if (o.data && o.data.ChangeAction) {
                if (o.data.ChangeAction === "JSONSearchCatalogProductByGtin") {
                    if (o.data.SearchString === products[0].gtin[0]) {
                        result.resolve(responseData);
                    } else if(o.data.SearchString === products[1].gtin[0]) {
                        result.resolve({
                            success: false,
                            error: 'ApiCallError'
                        });
                    } else {
                        result.resolve(errorResponseData);
                    }
                }
            } else {
                result.reject();
            }

            return result;
        };


        QUnit.ok(true, "=== Test Module: searchEbayByGtin ====");
        ebaytools.searchEbayByGtin({
            value: products[0].gtin[0],
            objectId: "123",
            onAfterSearch: function(result) {
                QUnit.equal(
                    result.length,
                    2,
                    "2 products found corresponding to the search GTIN '" + products[0].gtin[0] + "'."
                );
            }
        });

        ebaytools.searchEbayByGtin({
            value: products[1].gtin[0],
            objectId: "123",
            onAfterSearch: function(result) {
                QUnit.equal(
                    result,
                    false,
                    "No product found corresponding to the search GTIN '" + products[1].gtin[0] + "'."
                );
            }
        });

        setTimeout(function() {
            QUnit.start();
        }, 100);
    });
});